"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
Object.defineProperty(exports, "requireGql", {
  enumerable: true,
  get: function get() {
    return _requireGql.requireGql;
  }
});
exports.default = void 0;

var _path = require("path");

var _fs = require("fs");

var _core = require("@babel/core");

var _requireGql = require("./requireGql");

var resolve;

var _default = function _default(_ref) {
  var t = _ref.types;
  return {
    manipulateOptions(_ref2) {
      var resolveModuleSource = _ref2.resolveModuleSource;

      if (!resolve) {
        resolve = resolveModuleSource || _requireGql.defaultResolve;
      }
    },

    visitor: {
      ImportDeclaration: {
        exit(curPath, _ref3) {
          var babelPath = _ref3.file.opts.filename,
              nodePath = _ref3.opts.nodePath;
          var importNames = curPath.node.specifiers;
          var importPath = curPath.node.source.value;

          if (importPath.endsWith('.graphql') || importPath.endsWith('.gql')) {
            // Find the file, using node resolution/NODE_PATH if necessary.
            var fallbackPaths = nodePath ? nodePath.split(_path.delimiter) : process.env.NODE_PATH;
            var absPath = resolve(importPath, babelPath);
            if (!(0, _fs.existsSync)(absPath)) absPath = require.resolve(importPath, {
              paths: fallbackPaths
            }); // Analyze the file, returning either schema source (str) or a docs object containing ops.

            var result = (0, _requireGql.requireGql)(absPath, {
              resolve,
              nowrap: false
            });

            if (typeof result === 'string') {
              curPath.replaceWith(buildVariableAST(result, importNames[0].local.name));
            } else {
              var replacements = buildReplacements(result, importNames);
              replacements.length > 1 ? curPath.replaceWithMultiple(replacements) : curPath.replaceWith(replacements[0]);
            }
          }

          function buildReplacements(docs, specifiers) {
            return specifiers.map(function (_ref4) {
              var type = _ref4.type,
                  imported = _ref4.imported,
                  local = _ref4.local;

              switch (type) {
                case 'ImportDefaultSpecifier':
                  return buildVariableAST(docs.default, local.name);

                case 'ImportSpecifier':
                  return buildVariableAST(docs[imported.name] || docs.default, local.name);

                case 'ImportNamespaceSpecifier':
                  return buildVariableAST(docs, local.name);

                default:
                  throw new Error(`Unexpected import specifier type: ${type}`);
              }
            });
          }

          function buildVariableAST(graphqlAST, importName) {
            var _transformSync = (0, _core.transformSync)(`var ${importName} = ${JSON.stringify(graphqlAST)}`, {
              ast: true
            }),
                ast = _transformSync.ast;

            return ast.program.body[0];
          }
        }

      }
    }
  };
};

exports.default = _default;